part of 'profile_cubit.dart';

abstract class ProfileState extends Equatable {
  @override
  List<Object> get props => [];
}

class ProfileInitial extends ProfileState {
  @override
  String toString() {
    return 'ProfileInitial{}';
  }
}

class ProfileLoading extends ProfileState {
  @override
  String toString() => 'ProfileLoading';
}

class ValidateNameError extends ProfileState {
  final String? error;

  ValidateNameError(this.error);

  @override
  String toString() => 'ValidateNameError { error: $error }';
}

class ValidateEmailError extends ProfileState {
  final String? error;

  ValidateEmailError(this.error);

  @override
  String toString() => 'ValidateEmailError { error: $error }';
}

class ValidatePhoneNumberError extends ProfileState {
  final String? error;

  ValidatePhoneNumberError(this.error);

  @override
  String toString() => 'ValidatePhoneNumberError { error: $error }';
}

class ValidateShippingPhoneNumberError extends ProfileState {
  final String? error;

  ValidateShippingPhoneNumberError(this.error);

  @override
  String toString() => 'ValidateShippingPhoneNumberError { error: $error }';
}

class ProfileFailure extends ProfileState {
  final String error;

  ProfileFailure(this.error);

  @override
  String toString() => 'ProfileFailure { error: $error }';
}

class GetProfileSuccess extends ProfileState {
  final String? message;

  GetProfileSuccess({this.message});

  @override
  String toString() => 'GetProfileSuccess {message: $message}';
}

class UpdateProfileSuccess extends ProfileState {
  final String? message;

  UpdateProfileSuccess({this.message});

  @override
  String toString() => 'UpdateProfileSuccess {message: $message}';
}
